@@ -113,11 +113,17 @@ History:
 2011-02-17 ROwen    Document that addROWdgSet can take fewer widgets than values, but not more.
 2011-06-16 ROwen    Ditched obsolete "except (SystemExit, KeyboardInterrupt): raise" code
 2011-06-17 ROwen    Changed "type" to "msgType" in parsed message dictionaries to avoid conflict with builtin.
+2012-06-01 ROwen    Modified CmdVar cleanup as follows:
+                    - Do NOT remove keyVars (so the user can still read them)
+                    - Remove the time limit keyVar callback, if present
+                    - Use best effort to remove callbacks (do not raise an exception)
+2012-07-09 ROwen    Removed unused import in demo section.
+2012-07-18 ROwen    Modified to use RO.Comm.Generic.Timer.
+2012-11/29 ROwen    In CmdVar cast actor, cmdStr and abortStr to str to avoid unicode.
 """
 import sys
 import time
 import traceback
-import Tkinter
 import RO.AddCallback
 import RO.Alg
 import RO.CnvUtil
@@ -126,6 +132,11 @@ import RO.LangUtil
 import RO.PVT
 import RO.StringUtil
 import RO.SeqUtil
+import RO.Comm.Generic
+if RO.Comm.Generic.getFramework() is None:
+    print "Warning: RO.Comm.Generic framework not set; setting to tk"
+    RO.Comm.Generic.setFramework("tk")
+from RO.Comm.Generic import Timer
 
 # TypeDict translates message type characters to message categories
 # entries are: (meaning, category), where:
@@ -574,13 +585,16 @@ class KeyVar(RO.AddCallback.BaseMixin):
 
 class PVTKeyVar(KeyVar):
     """Position, velocity, time tuple for a given # of axes.
+    
+    To do: make regular callbacks optional for vel!=0 or remove entirely
+    and ask the user to implement this directly.
 
     Similar to KeyVar except:
     - The supplied keyword data is in the form:
         pos1, vel1, t1, pos2, vel2, t2..., pos<naxes>, vel<naxes>, t<naxes>
     - Values are PVTs
+    - The callback function is called once per second if velocity nonzero for any axis.
     """
-    _tkWdg = None
     def __init__(self,
         keyword,
         naxes=1,
@@ -597,9 +611,7 @@ class PVTKeyVar(KeyVar):
         **kargs)
 
         self._hasVel = False
-        if PVTKeyVar._tkWdg == None:
-            PVTKeyVar._tkWdg = Tkinter.Frame()
-        self._afterID = None
+        self._timer = Timer()
 
     def addPosCallback(self, callFunc, ind=0, callNow=True):
         """Similar to addIndexedCallback, but the call function
@@ -695,11 +707,10 @@ class PVTKeyVar(KeyVar):
     def _doCallbacks(self):
         """Call the callback functions.
         """
-        if self._afterID:
-            PVTKeyVar._tkWdg.after_cancel(self._afterID)
+        self._timer.cancel()
         KeyVar._doCallbacks(self)
         if self._hasVel:
-            self._afterID = PVTKeyVar._tkWdg.after(1000, self._doCallbacks)
+            self._timer.start(1.0, self._doCallbacks)
 
 class CmdVar(object):
     """Issue a command via the dispatcher and receive callbacks
@@ -746,14 +757,14 @@ class CmdVar(object):
         Also the time limit is a lower limit. The command is guaranteed to
         expire no sooner than this 
         """
-        self.cmdStr = cmdStr
-        self.actor = actor
+        self.cmdStr = str(cmdStr)
+        self.actor = str(actor)
         self.cmdID = None
         self.timeLim = timeLim
         self.description = description
         self.isRefresh = isRefresh
         self.timeLimKeyword = timeLimKeyword
-        self.abortCmdStr = abortCmdStr
+        self.abortCmdStr = str(abortCmdStr)
         self.keyVarDict = dict()
         if keyVars == None:
             keyVars = ()
@@ -929,16 +940,16 @@ class CmdVar(object):
                 self.maxEndTime += self.timeLim
 
     def _cleanup(self):
-        """Call when command is finished
-        to remove callbacks and avoid memory waste.
+        """Call when command is finished to remove callbacks and avoid wasting or leaking memory.
         """
         self.callTypesFuncList = []
         for keyVar in self.keyVars:
             try:
-                keyVar.removeCallback(self._keyVarCallback)
+                keyVar.removeCallback(self._keyVarCallback, doRaise=False)
             except ValueError:
                 pass
-        self.keyVars = []               
+        if self.timeLimKeyword:
+            self.removeCallback(self._checkForTimeLimKeyword, doRaise=False)
     
     def _setStartInfo(self, dispatcher, cmdID):
         """Called by the dispatcher when dispatching the command.
@@ -1089,9 +1100,9 @@ class KeyVarFactory(object):
 
 
 if __name__ == "__main__":
+    import Tkinter
     doBasic = True
     doFmt = True
-    import RO.KeyDispatcher
     import RO.Astro.Tm
     
     root = Tkinter.Tk()
